@@ -42,13 +42,78 @@ nodes.ClassDef._infer = infer_end
 nodes.FunctionDef._infer = infer_end
 nodes.Lambda._infer = infer_end
 nodes.Const._infer = infer_end
-nodes.List._infer = infer_end
-nodes.Tuple._infer = infer_end
-nodes.Dict._infer = infer_end
-nodes.Set._infer = infer_end
 nodes.Slice._infer = infer_end
 
 
+def infer_seq(self, context=None):
+    if not any(isinstance(e, nodes.Starred) for e in self.elts):
+        yield self
+    else:
+        values = _infer_seq(self, context)
+        new_seq = type(self)(self.lineno, self.col_offset, self.parent)
+        new_seq.postinit(values)
+        yield new_seq
+
+
+def _infer_seq(node, context=None):
+    """Infer all values based on _BaseContainer.elts"""
+    values = []
+
+    for elt in node.elts:
+        if isinstance(elt, nodes.Starred):
+            starred = helpers.safe_infer(elt.value, context)
+            if starred in (None, util.Uninferable):
+                raise exceptions.InferenceError(node=node,
+                                                context=context)
+            if not hasattr(starred, 'elts'):
+                raise exceptions.InferenceError(node=node,
+                                                context=context)
+            values.extend(_infer_seq(starred))
+        else:
+            values.append(elt)
+    return values
+
+
+nodes.List._infer = infer_seq
+nodes.Tuple._infer = infer_seq
+nodes.Set._infer = infer_seq
+
+
+def infer_map(self, context=None):
+    if not any(isinstance(k, nodes.DictUnpack) for k, _ in self.items):
+        yield self
+    else:
+        items = _infer_map(self, context)
+        new_seq = type(self)(self.lineno, self.col_offset, self.parent)
+        new_seq.postinit(list(items.items()))
+        yield new_seq
+
+
+def _infer_map(node, context):
+    """Infer all values based on Dict.items"""
+    values = {}
+    for name, value in node.items:
+        if isinstance(name, nodes.DictUnpack):
+            double_starred = helpers.safe_infer(value, context)
+            if double_starred in (None, util.Uninferable):
+                raise exceptions.InferenceError
+            if not isinstance(double_starred, nodes.Dict):
+                raise exceptions.InferenceError(node=node,
+                                                context=context)
+            values.update(_infer_map(double_starred, context))
+        else:
+            key = helpers.safe_infer(name, context=context)
+            value = helpers.safe_infer(value, context=context)
+            if any(elem in (None, util.Uninferable) for elem in (key, value)):
+                raise exceptions.InferenceError(node=node,
+                                                context=context)
+            values[key] = value
+    return values
+
+
+nodes.Dict._infer = infer_map
+
+
 def _higher_function_scope(node):
     """ Search for the first function which encloses the given
     scope. This can be used for looking up in that function's
@@ -66,6 +131,7 @@ def _higher_function_scope(node):
         current = current.parent
     if current and current.parent:
         return current.parent
+    return None
 
 def infer_name(self, context=None):
     """infer a Name: use name lookup rules"""
@@ -260,6 +326,7 @@ def infer_subscript(self, context=None):
         assigned = value.getitem(index_value, context)
     except (exceptions.AstroidTypeError,
             exceptions.AstroidIndexError,
+            exceptions.AttributeInferenceError,
             AttributeError) as exc:
         util.reraise(exceptions.InferenceError(node=self, error=exc,
                                                context=context))
@@ -424,7 +491,7 @@ def _is_not_implemented(const):
     return isinstance(const, nodes.Const) and const.value is NotImplemented
 
 
-def  _invoke_binop_inference(instance, opnode, op, other, context, method_name):
+def _invoke_binop_inference(instance, opnode, op, other, context, method_name):
     """Invoke binary operation inference on the given instance."""
     methods = dunder_lookup.lookup(instance, method_name)
     method = methods[0]
@@ -572,7 +639,7 @@ def _infer_binary_operation(left, right, binary_opnode, context, flow_factory):
                 yield util.Uninferable
                 return
 
-            # TODO(cpopa): since the inferrence engine might return
+            # TODO(cpopa): since the inference engine might return
             # more values than are actually possible, we decide
             # to return util.Uninferable if we have union types.
             if all(map(_is_not_implemented, results)):
@@ -618,12 +685,11 @@ def _infer_binop(self, context):
                 return
 
             try:
-                results = _infer_binary_operation(lhs, rhs, self, context, _get_binop_flow)
+                for result in _infer_binary_operation(lhs, rhs, self,
+                                                      context, _get_binop_flow):
+                    yield result
             except exceptions._NonDeducibleTypeHierarchy:
                 yield util.Uninferable
-            else:
-                for result in results:
-                    yield result
 
 
 @decorators.yes_if_nothing_inferred
@@ -660,12 +726,10 @@ def _infer_augassign(self, context=None):
                 return
 
             try:
-                results = _infer_binary_operation(lhs, rhs, self, context, _get_aug_flow)
+                for result in _infer_binary_operation(lhs, rhs, self, context, _get_aug_flow):
+                    yield result
             except exceptions._NonDeducibleTypeHierarchy:
                 yield util.Uninferable
-            else:
-                for result in results:
-                    yield result
 
 
 @decorators.path_wrapper
